@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2020 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,55 +34,38 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			'fields' => 'json',
 			'dynamic_hostid' => 'db hosts.hostid',
 			'content_width' => 'int32',
-			'content_height' => 'int32',
-			'only_footer' => 'in 1',
-			'period' => 'int32'
+			'content_height' => 'int32'
 		]);
 	}
 
 	protected function doAction() {
-		if ($this->getInput('only_footer', 0)) {
-			$this->setResponse(new CControllerResponseData([
-				'only_footer' => true,
-				'period_string' => $this->hasInput('period')
-					? ' ('.zbx_date2age(0, $this->getInput('period', 0)).')'
-					: '',
-				'user' => [
-					'debug_mode' => $this->getDebugMode()
-				]
-			]));
-
-			return;
-		}
-
 		$fields = $this->getForm()->getFieldsData();
 
 		$uniqueid = $this->getInput('uniqueid');
 		$edit_mode = (int) $this->getInput('edit_mode', 0);
 
-		$width = $this->getInput('content_width', '100');
-		$height = $this->getInput('content_height', '100');
+		$width = (int) $this->getInput('content_width', 100);
+		$height = (int) $this->getInput('content_height', 100);
 
 		$dataid = 'graph_'.$uniqueid;
 		$containerid = 'graph_container_'.$uniqueid;
 		$dynamic_hostid = $this->getInput('dynamic_hostid', 0);
-		$dashboardid = $this->getInput('dashboardid', 0);
 		$resourceid = null;
-		$profileIdx = 'web.dashbrd';
-		$profileIdx2 = $dashboardid;
+		$profileIdx = 'web.dashbrd.filter';
+		$profileIdx2 = $this->getInput('dashboardid', 0);
 		$unavailable_object = false;
 		$header_label = '';
 
 		if ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH && $fields['graphid']) {
 			$resource_type = SCREEN_RESOURCE_GRAPH;
-			$resourceid = $fields['graphid'];
+			$resourceid = reset($fields['graphid']);
 			$graph_dims = getGraphDims($resourceid);
 			$graph_dims['graphHeight'] = $height;
 			$graph_dims['width'] = $width;
 		}
 		elseif ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_SIMPLE_GRAPH && $fields['itemid']) {
 			$resource_type = SCREEN_RESOURCE_SIMPLE_GRAPH;
-			$resourceid = $fields['itemid'];
+			$resourceid = $fields['itemid'][0];
 			$graph_dims = getGraphDims();
 			$graph_dims['graphHeight'] = $height;
 			$graph_dims['width'] = $width;
@@ -93,14 +76,9 @@ class CControllerWidgetGraphView extends CControllerWidget {
 		}
 		$graph_dims['shiftYtop'] = CLineGraphDraw::DEFAULT_TOP_BOTTOM_PADDING;
 
-		// Prepare timeline details
-		$timeline = calculateTime([
+		$timeline = getTimeSelectorPeriod([
 			'profileIdx' => $profileIdx,
-			'profileIdx2' => $profileIdx2,
-			'updateProfile' => false,
-			'period' => null,
-			'stime' => null,
-			'isNow' => null
+			'profileIdx2' => $profileIdx2
 		]);
 
 		$time_control_data = [
@@ -109,8 +87,6 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			'objDims' => $graph_dims,
 			'loadSBox' => 0,
 			'loadImage' => 1,
-			'periodFixed' => CProfile::get($profileIdx.'.timelinefixed', 1),
-			'sliderMaximumTimePeriod' => ZBX_MAX_PERIOD,
 			'reloadOnAdd' => 1,
 			'onDashboard' => 1
 		];
@@ -122,47 +98,52 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			'timeline' => $timeline,
 			'resourcetype' => $resource_type,
 			'profileIdx' => $profileIdx,
-			'profileIdx2' => $profileIdx2,
-			'updateProfile' => false
+			'profileIdx2' => $profileIdx2
 		];
 
 		// Replace graph item by particular host item if dynamic items are used.
 		if ($fields['dynamic'] == WIDGET_DYNAMIC_ITEM && $dynamic_hostid && $resourceid) {
 			// Find same simple-graph item in selected $dynamic_hostid host.
 			if ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_SIMPLE_GRAPH) {
-				$items = get_same_item_for_host(['itemid' => $resourceid], [$dynamic_hostid]);
-				$item = reset($items);
-				$resourceid = ($item && array_key_exists('itemid', $item)) ? $item['itemid'] : null;
+				$src_items = API::Item()->get([
+					'output' => ['key_'],
+					'itemids' => $resourceid,
+					'webitems' => true
+				]);
 
-				if ($resourceid === null
-						|| !in_array($item['value_type'], [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64])) {
+				$items = API::Item()->get([
+					'output' => ['itemid', 'hostid', 'name', 'key_'],
+					'selectHosts' => ['name'],
+					'hostids' => $dynamic_hostid,
+					'filter' => [
+						'key_' => $src_items[0]['key_'],
+						'value_type' => [ITEM_VALUE_TYPE_FLOAT, ITEM_VALUE_TYPE_UINT64]
+					],
+					'webitems' => true
+				]);
+
+				$item = reset($items);
+				$resourceid = $items ? $item['itemid'] : null;
+
+				if ($resourceid === null) {
 					$unavailable_object = true;
-					$resourceid = null;
-				}
-				else {
-					$db_item = API::Item()->get([
-						'output' => [],
-						'selectHosts' => ['name'],
-						'itemids' => $item['itemid']
-					]);
-					$item['hosts'] = $db_item[0]['hosts'];
 				}
 			}
 			// Find requested host and change graph details.
 			elseif ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH) {
 				// get host
 				$hosts = API::Host()->get([
-					'hostids' => $dynamic_hostid,
-					'output' => ['hostid', 'name']
+					'output' => ['hostid', 'host', 'name'],
+					'hostids' => $dynamic_hostid
 				]);
 				$host = reset($hosts);
 
 				// get graph
 				$graph = API::Graph()->get([
-					'graphids' => $resourceid,
 					'output' => API_OUTPUT_EXTEND,
-					'selectHosts' => ['hostid'],
-					'selectGraphItems' => API_OUTPUT_EXTEND
+					'selectGraphItems' => API_OUTPUT_EXTEND,
+					'selectHosts' => [],
+					'graphids' => $resourceid
 				]);
 				$graph = reset($graph);
 
@@ -199,19 +180,25 @@ class CControllerWidgetGraphView extends CControllerWidget {
 							$graph['ymin_type'] = GRAPH_YAXIS_TYPE_CALCULATED;
 						}
 					}
-
-					$graph['hosts'] = $hosts;
 				}
 
 				if ($graph) {
-					// Search if there are any items available for this dynamic host.
-					$new_dynamic = getSameGraphItemsForHost(
-						$graph['gitems'],
-						$dynamic_hostid,
-						false
-					);
+					$graph['hosts'] = $hosts;
 
-					if (!$new_dynamic) {
+					// Search if there are any items available for this dynamic host.
+					$new_dynamic = getSameGraphItemsForHost($graph['gitems'], $dynamic_hostid, false);
+
+					if ($new_dynamic) {
+						// Add destination host data required by CMacrosResolver::resolveGraphPositionalMacros().
+						foreach ($new_dynamic as &$item) {
+							$item['hostid'] = $host['hostid'];
+							$item['host'] = $host['host'];
+						}
+						unset($item);
+
+						$graph['name'] = CMacrosResolverHelper::resolveGraphName($graph['name'], $new_dynamic);
+					}
+					else {
 						$unavailable_object = true;
 					}
 				}
@@ -241,9 +228,10 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			elseif ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH) {
 				// get graph, used below
 				$graph = API::Graph()->get([
-					'graphids' => $resourceid,
 					'output' => API_OUTPUT_EXTEND,
-					'selectHosts' => ['name']
+					'selectHosts' => ['name'],
+					'graphids' => $resourceid,
+					'expandName' => true
 				]);
 				$graph = reset($graph);
 
@@ -261,66 +249,76 @@ class CControllerWidgetGraphView extends CControllerWidget {
 				}
 
 				if ($resourceid) {
-					$graph_src = new CUrl('chart.php');
-					$graph_src->setArgument('itemids[]', $resourceid);
-					$graph_src->setArgument('width', $width);
-					$graph_src->setArgument('height', $height);
-					$graph_src->setArgument('legend', $fields['show_legend']);
+					$graph_src = (new CUrl('chart.php'))
+						->setArgument('itemids', [$resourceid])
+						->setArgument('width', $width)
+						->setArgument('height', $height)
+						->setArgument('legend', $fields['show_legend']);
 				}
 				else {
 					$graph_src = new CUrl('chart3.php');
 				}
 
-				$graph_src->setArgument('period', $timeline['period']);
-				$graph_src->setArgument('stime', $timeline['stime']);
-				$graph_src->setArgument('isNow', $timeline['isNow']);
+				$graph_src
+					->setArgument('from', $timeline['from'])
+					->setArgument('to', $timeline['to']);
 
 				$item = CMacrosResolverHelper::resolveItemNames([$item])[0];
 				$header_label = $item['hosts'][0]['name'].NAME_DELIMITER.$item['name_expanded'];
 			}
 			elseif ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH) {
 				$graph_src = '';
-				$header_label = $graph['hosts'][0]['name'].NAME_DELIMITER.$graph['name'];
+
+				if (count($graph['hosts']) == 1 || $fields['dynamic'] == WIDGET_DYNAMIC_ITEM && $dynamic_hostid != 0) {
+					$header_label = $graph['hosts'][0]['name'].NAME_DELIMITER.$graph['name'];
+				}
+				else {
+					$header_label = $graph['name'];
+				}
 
 				if ($fields['dynamic'] == WIDGET_DYNAMIC_ITEM && $dynamic_hostid && $resourceid) {
-					$chart_file = ($graph['graphtype'] == GRAPH_TYPE_PIE || $graph['graphtype'] == GRAPH_TYPE_EXPLODED)
-						? 'chart7.php'
-						: 'chart3.php';
-
-					$graph_src = new CUrl($chart_file);
-
-					foreach ($graph as $name => $value) {
-						if ($name === 'width' || $name === 'height') {
-							continue;
-						}
-						$graph_src->setArgument($name, $value);
+					if ($graph['graphtype'] == GRAPH_TYPE_PIE || $graph['graphtype'] == GRAPH_TYPE_EXPLODED) {
+						$graph_src = (new CUrl('chart7.php'))
+							->setArgument('name', $host['name'].NAME_DELIMITER.$graph['name'])
+							->setArgument('graphtype', $graph['graphtype'])
+							->setArgument('graph3d', $graph['show_3d']);
+					}
+					else {
+						$graph_src = (new CUrl('chart3.php'))
+							->setArgument('name', $host['name'].NAME_DELIMITER.$graph['name'])
+							->setArgument('ymin_type', $graph['ymin_type'])
+							->setArgument('ymax_type', $graph['ymax_type'])
+							->setArgument('ymin_itemid', $graph['ymin_itemid'])
+							->setArgument('ymax_itemid', $graph['ymax_itemid'])
+							->setArgument('showworkperiod', $graph['show_work_period'])
+							->setArgument('showtriggers', $graph['show_triggers'])
+							->setArgument('graphtype', $graph['graphtype'])
+							->setArgument('yaxismin', $graph['yaxismin'])
+							->setArgument('yaxismax', $graph['yaxismax'])
+							->setArgument('percent_left', $graph['percent_left'])
+							->setArgument('percent_right', $graph['percent_right']);
 					}
 
 					$new_graph_items = getSameGraphItemsForHost($graph['gitems'], $dynamic_hostid, false);
 
-					foreach ($new_graph_items as $new_graph_item) {
+					foreach ($new_graph_items as &$new_graph_item) {
 						unset($new_graph_item['gitemid'], $new_graph_item['graphid']);
-
-						foreach ($new_graph_item as $name => $value) {
-							$graph_src->setArgument('items['.$new_graph_item['itemid'].']['.$name.']', $value);
-						}
 					}
+					unset($new_graph_item);
 
-					$graph_src->setArgument('name', $host['name'].NAME_DELIMITER.$graph['name']);
+					$graph_src->setArgument('items', $new_graph_items);
 				}
 
 				if ($graph_dims['graphtype'] == GRAPH_TYPE_PIE || $graph_dims['graphtype'] == GRAPH_TYPE_EXPLODED) {
 					if ($fields['dynamic'] == WIDGET_SIMPLE_ITEM || $graph_src === '') {
-						$graph_src = new CUrl('chart6.php');
-						$graph_src->setArgument('graphid', $resourceid);
+						$graph_src = (new CUrl('chart6.php'))
+							->setArgument('graphid', $resourceid)
+							->setArgument('graph3d', $graph['show_3d']);
 					}
-
-					$timeline['starttime'] = date(TIMESTAMP_FORMAT, get_min_itemclock_by_graphid($resourceid));
 				}
 				else {
 					if ($fields['dynamic'] == WIDGET_SIMPLE_ITEM || $graph_src === '') {
-						$graph_src = new CUrl('chart2.php');
-						$graph_src->setArgument('graphid', $resourceid);
+						$graph_src = (new CUrl('chart2.php'))->setArgument('graphid', $resourceid);
 					}
 
 					if (!$edit_mode) {
@@ -328,21 +326,17 @@ class CControllerWidgetGraphView extends CControllerWidget {
 					}
 				}
 
-				$graph_src->setArgument('width', $width);
-				$graph_src->setArgument('height', $height);
-				$graph_src->setArgument('legend', ($fields['show_legend'] == 1 && $graph['show_legend'] == 1) ? 1 : 0);
-				$graph_src->setArgument('period', $timeline['period']);
-				$graph_src->setArgument('stime', $timeline['stime']);
-				$graph_src->setArgument('isNow', $timeline['isNow']);
-
-				if ($graph_dims['graphtype'] == GRAPH_TYPE_PIE || $graph_dims['graphtype'] == GRAPH_TYPE_EXPLODED) {
-					$graph_src->setArgument('graph3d', $graph['show_3d']);
-				}
+				$graph_src
+					->setArgument('width', $width)
+					->setArgument('height', $height)
+					->setArgument('legend', ($fields['show_legend'] && $graph['show_legend']) ? 1 : 0)
+					->setArgument('from', $timeline['from'])
+					->setArgument('to', $timeline['to']);
 			}
 
-			$graph_src->setArgument('updateProfile', false);
-			$graph_src->setArgument('profileIdx', $profileIdx);
-			$graph_src->setArgument('profileIdx2', $profileIdx2);
+			$graph_src
+				->setArgument('profileIdx', $profileIdx)
+				->setArgument('profileIdx2', $profileIdx2);
 
 			if ($graph_dims['graphtype'] != GRAPH_TYPE_PIE && $graph_dims['graphtype'] != GRAPH_TYPE_EXPLODED) {
 				$graph_src->setArgument('outer', '1');
@@ -350,16 +344,32 @@ class CControllerWidgetGraphView extends CControllerWidget {
 
 			$graph_src->setArgument('widget_view', '1');
 			$time_control_data['src'] = $graph_src->getUrl();
+
+			if ($fields['source_type'] == ZBX_WIDGET_FIELD_RESOURCE_GRAPH) {
+				$item_graph_url = (new CUrl('zabbix.php'))
+					->setArgument('action', 'charts.view')
+					->setArgument('view_as', HISTORY_GRAPH)
+					->setArgument('filter_search_type', ZBX_SEARCH_TYPE_STRICT)
+					->setArgument('filter_graphids', [$resourceid])
+					->setArgument('filter_set', '1');
+			}
+			else {
+				$item_graph_url = (new CUrl('history.php'))->setArgument('itemids', [$resourceid]);
+			}
+			$item_graph_url
+				->setArgument('from', $timeline['from'])
+				->setArgument('to', $timeline['to']);
 		}
 
 		$response = [
-			'name' => $this->getInput('name', ''),
+			'name' => $this->getInput('name', $header_label),
 			'graph' => [
 				'dataid' => $dataid,
 				'containerid' => $containerid,
 				'timestamp' => time(),
 				'unavailable_object' => $unavailable_object
 			],
+			'item_graph_url' => $unavailable_object ? '' : $item_graph_url,
 			'widget' => [
 				'uniqueid' => $uniqueid,
 				'initial_load' => (int) $this->getInput('initial_load', 0),
@@ -367,18 +377,11 @@ class CControllerWidgetGraphView extends CControllerWidget {
 			'time_control_data' => $time_control_data,
 			'timeline' => $timeline,
 			'fs_data' => $fs_data,
-			'dashboardid' => $dashboardid,
-			'only_footer' => false,
 			'user' => [
 				'debug_mode' => $this->getDebugMode()
 			]
 		];
 
-		if ($response['name'] === '') {
-			$response['name'] = $header_label;
-			$response['period_string'] = ' ('.zbx_date2age(0, $timeline['period']).')';
-		}
-
 		$this->setResponse(new CControllerResponseData($response));
 	}
 }
